cmake_minimum_required(VERSION 3.25)
project(timbreIDLib)
set(LIB_DIR ${PD4WEB_EXTERNAL_DIR}/${PROJECT_NAME})

include_directories(${LIB_DIR}/include)

# ╭──────────────────────────────────────╮
# │                 FFTW                 │
# ╰──────────────────────────────────────╯
cmake_policy(SET CMP0135 NEW)
option(BUILD_SHARED_LIBS OFF)
option(BUILD_TESTS OFF)
set(ENABLE_FLOAT ON)
set(FFTW3_FILE ${CMAKE_BINARY_DIR}/fftw-3.3.10.tar.gz)
if(NOT EXISTS ${FFTW3_FILE})
  message(STATUS "Downloading FFTW3")
  file(DOWNLOAD https://www.fftw.org/fftw-3.3.10.tar.gz ${FFTW3_FILE})
endif()

file(ARCHIVE_EXTRACT INPUT ${CMAKE_BINARY_DIR}/fftw-3.3.10.tar.gz DESTINATION
     ${CMAKE_BINARY_DIR}/)
add_subdirectory("${CMAKE_BINARY_DIR}/fftw-3.3.10" EXCLUDE_FROM_ALL)
set_target_properties(fftw3f PROPERTIES LIBRARY_OUTPUT_DIRECTORY
                                       ${CMAKE_SOURCE_DIR})
set_target_properties(fftw3f PROPERTIES POSITION_INDEPENDENT_CODE ON)
include_directories("${CMAKE_BINARY_DIR}/fftw-3.3.10/api")

# ╭──────────────────────────────────────╮
# │               OBJECTS                │
# ╰──────────────────────────────────────╯

add_library(tIDLib STATIC "${LIB_DIR}/src/tIDLib.c")

pd_add_external(attackTime "${LIB_DIR}/src/attackTime.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(attackTime~ "${LIB_DIR}/src/attackTime~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(autoCorrPitch "${LIB_DIR}/src/autoCorrPitch.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(autoCorrPitch~ "${LIB_DIR}/src/autoCorrPitch~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(bark "${LIB_DIR}/src/bark.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(bark2freq "${LIB_DIR}/src/bark2freq.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpec "${LIB_DIR}/src/barkSpec.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecBrightness "${LIB_DIR}/src/barkSpecBrightness.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecBrightness~ "${LIB_DIR}/src/barkSpecBrightness~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecCentroid "${LIB_DIR}/src/barkSpecCentroid.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecCentroid~ "${LIB_DIR}/src/barkSpecCentroid~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecFlatness "${LIB_DIR}/src/barkSpecFlatness.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecFlatness~ "${LIB_DIR}/src/barkSpecFlatness~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecFlux "${LIB_DIR}/src/barkSpecFlux.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecFlux~ "${LIB_DIR}/src/barkSpecFlux~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecIrregularity "${LIB_DIR}/src/barkSpecIrregularity.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecIrregularity~ "${LIB_DIR}/src/barkSpecIrregularity~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecKurtosis "${LIB_DIR}/src/barkSpecKurtosis.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecKurtosis~ "${LIB_DIR}/src/barkSpecKurtosis~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecRolloff "${LIB_DIR}/src/barkSpecRolloff.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecRolloff~ "${LIB_DIR}/src/barkSpecRolloff~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecSkewness "${LIB_DIR}/src/barkSpecSkewness.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecSkewness~ "${LIB_DIR}/src/barkSpecSkewness~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecSlope "${LIB_DIR}/src/barkSpecSlope.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecSlope~ "${LIB_DIR}/src/barkSpecSlope~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecSpread "${LIB_DIR}/src/barkSpecSpread.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpecSpread~ "${LIB_DIR}/src/barkSpecSpread~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(barkSpec~ "${LIB_DIR}/src/barkSpec~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(bark~ "${LIB_DIR}/src/bark~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(bfcc "${LIB_DIR}/src/bfcc.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(bfcc~ "${LIB_DIR}/src/bfcc~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(bin2freq "${LIB_DIR}/src/bin2freq.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(binWrangler "${LIB_DIR}/src/binWrangler.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(cepstrum "${LIB_DIR}/src/cepstrum.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(cepstrum~ "${LIB_DIR}/src/cepstrum~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(cepstrumPitch "${LIB_DIR}/src/cepstrumPitch.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(cepstrumPitch~ "${LIB_DIR}/src/cepstrumPitch~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(chroma "${LIB_DIR}/src/chroma.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(chroma~ "${LIB_DIR}/src/chroma~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(dct "${LIB_DIR}/src/dct.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(dct~ "${LIB_DIR}/src/dct~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(energy "${LIB_DIR}/src/energy.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(energy~ "${LIB_DIR}/src/energy~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(energyEntropy "${LIB_DIR}/src/energyEntropy.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(energyEntropy~ "${LIB_DIR}/src/energyEntropy~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(mel2freq "${LIB_DIR}/src/mel2freq.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(melSpec "${LIB_DIR}/src/melSpec.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(melSpec~ "${LIB_DIR}/src/melSpec~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(mfcc "${LIB_DIR}/src/mfcc.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(mfcc~ "${LIB_DIR}/src/mfcc~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(minSample "${LIB_DIR}/src/minSample.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(minSample~ "${LIB_DIR}/src/minSample~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(minSampleDelta "${LIB_DIR}/src/minSampleDelta.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(minSampleDelta~ "${LIB_DIR}/src/minSampleDelta~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(nearestPoint "${LIB_DIR}/src/nearestPoint.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(peakSample "${LIB_DIR}/src/peakSample.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(peakSample~ "${LIB_DIR}/src/peakSample~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(phaseSpec "${LIB_DIR}/src/phaseSpec.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(phaseSpec~ "${LIB_DIR}/src/phaseSpec~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(sampleBuffer~ "${LIB_DIR}/src/sampleBuffer~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specBrightness "${LIB_DIR}/src/specBrightness.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specBrightness~ "${LIB_DIR}/src/specBrightness~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specCentroid "${LIB_DIR}/src/specCentroid.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specCentroid~ "${LIB_DIR}/src/specCentroid~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specFlatness "${LIB_DIR}/src/specFlatness.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(featureAccum "${LIB_DIR}/src/featureAccum.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(featureDelta "${LIB_DIR}/src/featureDelta.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(featureNorm "${LIB_DIR}/src/featureNorm.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(freq2bark "${LIB_DIR}/src/freq2bark.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(freq2bin "${LIB_DIR}/src/freq2bin.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(freq2mel "${LIB_DIR}/src/freq2mel.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(harmonicRatio "${LIB_DIR}/src/harmonicRatio.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(harmonicRatio~ "${LIB_DIR}/src/harmonicRatio~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(magSpec "${LIB_DIR}/src/magSpec.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(magSpec~ "${LIB_DIR}/src/magSpec~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(maxSample "${LIB_DIR}/src/maxSample.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(maxSample~ "${LIB_DIR}/src/maxSample~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(maxSampleDelta "${LIB_DIR}/src/maxSampleDelta.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(maxSampleDelta~ "${LIB_DIR}/src/maxSampleDelta~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specFlatness~ "${LIB_DIR}/src/specFlatness~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specFlux "${LIB_DIR}/src/specFlux.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specFlux~ "${LIB_DIR}/src/specFlux~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specHarmonicity "${LIB_DIR}/src/specHarmonicity.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specHarmonicity~ "${LIB_DIR}/src/specHarmonicity~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specIrregularity "${LIB_DIR}/src/specIrregularity.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specIrregularity~ "${LIB_DIR}/src/specIrregularity~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specKurtosis "${LIB_DIR}/src/specKurtosis.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specKurtosis~ "${LIB_DIR}/src/specKurtosis~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specRolloff "${LIB_DIR}/src/specRolloff.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specRolloff~ "${LIB_DIR}/src/specRolloff~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specSkewness "${LIB_DIR}/src/specSkewness.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specSkewness~ "${LIB_DIR}/src/specSkewness~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specSlope "${LIB_DIR}/src/specSlope.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specSlope~ "${LIB_DIR}/src/specSlope~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specSpread "${LIB_DIR}/src/specSpread.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(specSpread~ "${LIB_DIR}/src/specSpread~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(tempo~ "${LIB_DIR}/src/tempo~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(tID_fft "${LIB_DIR}/src/tID_fft.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(tID_fft~ "${LIB_DIR}/src/tID_fft~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(tID_mean "${LIB_DIR}/src/tID_mean.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(tID_std "${LIB_DIR}/src/tID_std.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(timbreID "${LIB_DIR}/src/timbreID.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(tabletool "${LIB_DIR}/src/tabletool.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(waveSlope "${LIB_DIR}/src/waveSlope.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(waveSlope~ "${LIB_DIR}/src/waveSlope~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(waveNoise "${LIB_DIR}/src/waveNoise.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(waveNoise~ "${LIB_DIR}/src/waveNoise~.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(zeroCrossing "${LIB_DIR}/src/zeroCrossing.c" LINK_LIBRARIES fftw3 tIDLib)
pd_add_external(zeroCrossing~ "${LIB_DIR}/src/zeroCrossing~.c" LINK_LIBRARIES fftw3 tIDLib)
